#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "linkedlist.h"

int compare(const void *s1, const void *s2) {
  return strcmp((const char*)s1, (const char*)s2);
}

int main(int argc, char **argv) {
  linkedlist *ll = NULL;
  int i, retVal;

  /* User must pass at least one strings. */
  if (argc < 2) {
    printf("%s: requires at least one string.\n",argv[0]);
    return ERROR;
  }

  /* Malloc space for the linkedlist and ensure the list was initialized correctly. */
  ll = (linkedlist *)malloc(sizeof(linkedlist));
  if (initLL(ll) != SUCCESS) {
    return ERROR;
  }

  /* For each user given arguments, add to the list only if it was not already present.
   * (preserve uniqueness in the list.) */
  for (i = 1; i < argc; ++i) {
    if ((retVal = findNode(ll, compare, argv[i])) == ERROR) {
      addNode(ll, compare, argv[i]);
    }
    if (retVal == -1) {
      destroyLL(ll,NULL);
      free(ll);
      ll = NULL;
      return ERROR;
    }
  }


  /* Print out how many unique items are in the list. */
  printf("There are %lu items in the list.\n",ll->count);


  /* Destroy the list. */
  destroyLL(ll,NULL);
  free(ll); /* COMMENT ME FOR MEMORY LEAK */
  /* free(ll); UNCOMMENT ME FOR SEGFAULT, DOUBLE FREE */
  ll = NULL; 
  
  return SUCCESS;
}
