#include <stdlib.h>
#include <stdio.h>
#include "linkedlist.h"

int initLL (linkedlist *list){
/*check if null*/
	if(!list) return ERROR;
	
	/*initilize without any nodes*/
	list->head = list->tail = NULL;
	list->count = 0;

	return SUCCESS;
}



static int initNode (node **new, void *data, node *next, node *prev){
	/*check if data exists*/
	if(!data) return ERROR;
/*initilize node*/
	*new = (node *) malloc (sizeof(node));
	(*new)->data = data;
	(*new)->next = next;
	(*new)->prev = prev;
	return SUCCESS;
}

int destroyLL (linkedlist *list, int (*destroy)(void **)){
	/*index node is considered bacon so it can be given away for free
	 * much like Kesden uses Zombie*/
	node *bacon = list->head;
	/*check if list is NULL*/
	if(!list) return ERROR;

	/*for each node... remove it*/
	while (list->count){
		/*set new head to next node*/
		list->head = bacon->next;
		if(bacon->next != NULL){
			/*point bacon's next's previous to NULL*/
			bacon->next->prev = bacon->prev;
		}else list->tail = NULL;
		/*decrement counter*/
		list->count--;
    /* If user allocated, DESTROY EVIDENCE OF BACON'S DATA! */
		if(destroy != NULL){
      (*destroy)(&(bacon->data));
    }
		/* FREE BACON?! IT'S GOING FAST!
		 * WHY? BECAUSE EVERYONE LOVES FREE BACON!! */	
		free(bacon);
		/*bacon is head*/
		bacon = list->head;
	}
	
	return SUCCESS;
}

int findNNode (linkedlist *list,unsigned int index, void **data){
	/*index node is considered bacon so it can be given away for free
	 * much like Kesden uses Zombie*/
	node *bacon = list->head;
	unsigned int i = 0;
	/*check if list exists*/
	if(!list || !bacon) return ERROR;
	/*is index out of bounds*/
	if(index > list->count) return ERROR;
	/*iterate up to the Nth node*/
	while(i < index){
		bacon=bacon->next;
		i++;
	}
	/*return the data held in the Nth node*/
	*data = bacon->data;
	return SUCCESS;
}

int findNode (linkedlist *list, int (*compare)(const void *, const void *), void *data){
	/*index node is considered bacon so it can be given away for free
	 * much like Kesden uses Zombie*/
	node *bacon = list->tail;
	/*Check if values are NULL, if so than return -1 for error because
	 * 1 is associated to not found*/
	if((!compare) || !list) return -1;
	if(!data) return ERROR;
	/*start from tail and head backwards through sorted list*/
	while(bacon != NULL && (*compare)(bacon->data, data) > 0)
		bacon = bacon->prev;
	/*if bacon isn't NULL and data was found at node bacon
	 * return success = 0*/
	if(bacon != NULL && (*compare)(bacon->data, data) == 0)
		return SUCCESS;
	/*else it wasn't found return not found = 1*/
	return ERROR;
}

int addNode (linkedlist *list,int (*compare)(const void *, const void *) , void *data){
	node *new;
	/*index node is considered bacon so it can be given away for free
	 * much like Kesden uses Zombie*/
	node *bacon = list->tail;
	/*check if they are null/Exists*/
	if((!compare) || !list || !data) return ERROR;

	/*if there are no previous nodes*/
	if(!list->count){
		initNode(&new, data, list->head, list->head);
		list->head = list->tail = new;
		list->count++;
		return SUCCESS;
	}
	/*consider an already sorted list, approach from the end
	 * comparing until you reach the node's between which 
	 * one places the new node to be sandwiched between bacon
	 * and bacon's next (perhaps we should call this lettuce
	 * and the new node tomato)*/
	while(bacon != NULL && (*compare)(bacon->data, data)>0)
		bacon = bacon->prev;
	/* if head, init node with previous as NULL and next as old head 
	 * else use init node between bacon and bacon's next */
	if(bacon == NULL){
		initNode(&new, data, list->head, NULL);
		/*if bacon is NULL than the new node becomes
		 * the head of the list and the old head's previous 
		 * becomes the new node */
		list->head->prev = new;
		list->head = new;
	}else{
		initNode(&new, data, bacon->next, bacon);
		/* if tail, set as tail else point next node's previous to new
		 * and bacon's next to new */
		if(bacon->next == NULL)
			list->tail = new;
		else
			bacon->next->prev = new;
		bacon->next = new;
	
	}
	/*increment the count*/
	list->count++;
	return SUCCESS;
}

int removeNode (linkedlist *list, int (*compare)(const void *, const void *), void *data, void **rData){
	/*index node is considered bacon so it can be given away for free
	 * much like Kesden uses Zombie*/
	node *bacon = list->tail;
	if(!list || (!compare) || !data) return ERROR;
	/*start from end and work through sorted list*/
	while(bacon != NULL && (*compare)(bacon->data, data)>0)
		bacon = bacon->prev;
	/*if bacon is NOT NULL and the data is found, remove bacon*/
	if(bacon != NULL && (*compare)(bacon->data, data) == 0){
		/* if tail, set as tail else point next node to 
		 * bacon's previous
		 * if bacon is head node, bacon's next's previous points 
		 * at NULL, which is fine */
		if(bacon->next == NULL)
			list->tail = bacon->prev;
		else
			bacon->next->prev = bacon->prev;
		/* if head, set as head else point previous node to 
		 * bacon's next
		 * if bacon is tail, bacon's previous' next points at
		 * NULL, which is fine */
		if(bacon->prev == NULL)
			list->head = bacon->next;
		else
			bacon->prev->next = bacon->next;
		/* decrement count, store removed node's data */
		list->count--;
		*rData = (bacon->data);
		/* FREE BACON?! IT'S GOING FAST!
		 * WHY? BECAUSE EVERYONE LOVES FREE BACON!! */
		free(bacon);
		return SUCCESS;
	}
	/*else, it wasn't found return error*/
	return ERROR;
}

int getNode(linkedlist *list, int (*compare)(const void *,const  void *), void *data, void **rData){
	/*index node is considered bacon so it can be given away for free
	 * much like Kesden uses Zombie*/
	node *bacon = list->tail;
	/*Check if values are NULL, if so than return 1 for error*/
	if((!compare) || !list || !data) return ERROR;
	/*start from tail and head backwards through sorted list*/
	while(bacon != NULL && (*compare)(bacon->data, data) > 0)
		bacon = bacon->prev;
	/*if bacon isn't NULL and data was found at node bacon
	 * set rData to point to bacon's Data
	 * return success = 0*/
	if(bacon != NULL && (*compare)(bacon->data, data) == 0){
		*rData = (bacon->data);
		return SUCCESS;
	}
	/*else it wasn't found return not found = 1*/
	return ERROR;
}

int getNodeIndex (linkedlist *list, int (*compare)(const void *, const void *), void *data, unsigned int *index){
	/*index node is considered bacon so it can be given away for free
	 * much like Kesden uses Zombie*/
	node *bacon = list->tail;
	*index = list->count - 1;
	/*check if list exists*/
	if((!compare) || !list || !bacon || !data) return ERROR;
	/*iterate up to the data node*/
	while(bacon != NULL && (*compare)(bacon->data, data) > 0){
		bacon=bacon->next;
		(*index)--;
	}
	/*if bacon isn't NULL and data was found at node bacon
	 * set rData to point to bacon's Data
	 * return success = 0*/
	if(bacon != NULL && (*compare)(bacon->data, data) == 0){
		return SUCCESS;
	}
	/*else it wasn't found return not found = 1*/
	return ERROR;
}


