#ifndef LINKEDLIST_H
#define LINKEDLIST_H

struct node_t;

typedef struct node_t{
	void *data;
	struct node_t *next;
	struct node_t *prev;
} node;

typedef struct {
	node *head;
	node *tail;
	unsigned long count;
} linkedlist;

/* Takes a user allocated chunk of memory and initializes it as a linkedlist.*/
int initLL (linkedlist *list);

/* Destroys every node in the linkedlist including the data using the destroy function.
 * If the destroy function is NULL then the data is not freed. This option exists for
 * nondynamiclaly allocated space. The user must deallocate the memory for the list afterwards. */
int destroyLL (linkedlist *list, int(*destroy)(void **));

/* Adds the data to the list based on the compare function. The compare function
 * must be specified. Duplicates are still added to the list. */
int addNode (linkedlist *list,int (*compare)(const void *, const void *), void *data);

/* Removes the data from the list based on the compare function. The compare function
 * must be specified. In the event of duplicates, the newest node is removed. */
int removeNode (linkedlist *list, int (*compare)(const void *, const void *), void *data, void **rData);

/* Check for SUCCESS if the item was found in the list.
 * ERROR means it was not found in the list and -1 means you have invalid arguments. */
int findNode (linkedlist *list, int (*compare)(const void *, const void *), void *data);

/* If SUCCESS, then the data's pointer shall be passed in data */
int findNNode (linkedlist *list, unsigned int index, void **data);

/* Upon success, return the pointer to the data in rData. */
int getNode (linkedlist *list, int (*compare)(const void *, const void *), void *data, void **rData);

/* Upon SUCCESS, returns the index of the node for the item given in data. */
int getNodeIndex (linkedlist *list, int (*compare)(const void *, const void *), void *data, unsigned int *index);
#define SUCCESS	0
#define ERROR	1


#endif
